<#	
	.NOTES
	===========================================================================
	 Created on:   	08.04.2014 18:48
	 Created by:   	Andreas Nick
	 Organization: 	Nick Informationstechnik GmbH
	===========================================================================
	.DESCRIPTION
		Powershell XML signing example
#>

[void][reflection.assembly]::Load('System.Security, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a')


[xml]  $ExampleXML = `
"<?xml version=""1.0"" encoding=""utf-8""?>
<root>
<so>
<user name = ""Andreas"">thats me</user>
<user name = ""Peter"">I like this</user>
</so>
</root>"

function Write-Xml2Disk {
    Param (
    [String] $path
    )
    $utf = New-Object system.Text.UTF8Encoding -ArgumentList $false
    $xmltw = New-Object System.Xml.XmlTextWriter -ArgumentList @($Path, $utf)
    $myxml.WriteTo($xmltw);
    $xmltw.Close();
}

function Create-RSAKey {
    
    $cspParams = New-Object System.Security.Cryptography.CspParameters -ArgumentList 1
    #$cspParams.KeyContainerName = "RSA CONTAINER"
    $cspParams.Flags = [System.Security.Cryptography.CspProviderFlags]::UseArchivableKey
    $cspParams.KeyNumber = [int][System.Security.Cryptography.KeyNumber]::Exchange
    #$cspParams.KeyPassword = convertto-securestring "P@ssW0rD!" -asplaintext -force
    [System.Security.Cryptography.RSACryptoServiceProvider] $rsaKey = New-Object System.Security.Cryptography.RSACryptoServiceProvider -ArgumentList @(2048,$cspParams)
    
    #Nicht dauerhaft speichern
    $rsaKey.PersistKeyInCsp = $false;
    
    return $rsaKey
}

function Export-RSAKeyPair {
    Param (
        [String] $Path,
        [system.Security.Cryptography.RSA] $rsaKey
    )
    
    $privateKey = [xml]$rsaKey.ToXmlString($true);
    $publicKey = [xml]$rsaKey.ToXmlString($false);
    
    $privateKey.Save("$ScriptDirectory\PrivateKey.xml")
    $PublicKey.Save("$ScriptDirectory\PublicKey.xml")
}


function Verify-XmlSignature {
    Param (
    [xml] $checkxml,
    [system.Security.Cryptography.RSA] $Key
    )
    
    [System.Security.Cryptography.Xml.SignedXml] $signedXml = New-Object System.Security.Cryptography.Xml.SignedXml -ArgumentList $checkxml
    $XmlNodeList = $checkxml.GetElementsByTagName("Signature")
	$signedXml.LoadXml([System.Xml.XmlElement] ($XmlNodeList[0]))
    $check = $signedXml.CheckSignature($key)
	return $check
}

function Sign-XML {
    Param (
        [xml] $xml,
        [system.Security.Cryptography.RSA] $rsaKey
    )
    
    [System.Security.Cryptography.xml.SignedXml]  $signedXml = $NULL
    $signedXml = New-Object System.Security.Cryptography.Xml.SignedXml -ArgumentList $xml
    $signedXml.SigningKey = $rsaKey
    $referenz = New-Object System.Security.Cryptography.Xml.Reference
    $referenz.Uri = "" 
    $env = New-Object System.Security.Cryptography.Xml.XmlDsigEnvelopedSignatureTransform
    $referenz.AddTransform($env);
    $signedXml.AddReference($referenz)
    $signedXml.ComputeSignature()
    [System.Xml.XmlElement] $xmlSignatur = $signedXml.GetXml()
    $xml.DocumentElement.AppendChild($xml.ImportNode($xmlSignatur, $true))
    
    if ($xml.FirstChild -is [system.xml.XmlDeclaration]) {
        $xml.RemoveChild($xml.FirstChild);
    }
    $xml
}

$Path = Split-Path $script:MyInvocation.MyCommand.Path
#Write-Host $Path

$Key = Create-RSAKey 
Sign-XML -xml $ExampleXML -rsaKey $Key
$ExampleXML.Save("$Path\myxml.xml")
$Checked = Verify-XmlSignature -checkxml $ExampleXML -Key $Key

Write-Host "Signature check status : " $Checked











